define(['jquery', 'lodash', 'history', 'js/demoloader', 'templates/index'], function ($, _, History, loadDemos, sections) {
    'use strict';

    var titlePrefix = "Blue Sky UX Guide: ",
        $allRootNavItems = $('ul#root-navigation li'),
        $sectionNavigation = $('#section-navigation-root'),
        $content = $('#content'),
        currentSection, currentSectionName;

    sections.search = {
        title : 'Search Results',
        titles : {},
        topics : {}
    };

    var that = {};

    // Get an object of arguments provided in URL
    function parseURL(url) {
        var data = {},
            root = url.replace(/.*\?/, '').replace(/\/$/, '').split('#'),
            hash = root[1],
            sanitized = root[0] || [],
            segments = sanitized.split('&'),
            splitSegment;

        _.each(segments || [], function (segment) {
            if (!segment) {
                return;
            }

            splitSegment = segment.split('=');
            data[splitSegment[0]] = splitSegment[1];
        });

        if (hash && hash !== '') {
            data.anchor = hash;
        }

        return data;
    }

    function history(func, section, topic, subtopic, anchor) {
        History[func]({section:section, topic:topic, subtopic:subtopic}, null, '?section=' + section + (topic ? '&topic=' + topic : '') + (subtopic ? '&subtopic=' + subtopic : '') + (anchor ? '#' + anchor : ''));
    }

    function scrollTo($anchor) {
        $('html, body').animate({ scrollTop : $anchor.jquery ? $anchor.offset().top - 45 : $anchor /*top header*/ }, 0);
    }

    function loadSectionMenuResponse(section, clearContent, menu) {
        //$sectionNavigation.removeClass('span1').addClass('span2');
        if (clearContent) {
            $content.empty();
        }

        $allRootNavItems.removeClass('active');
        $allRootNavItems.find('a[data-section='+section+']').parent('li').addClass('active');

        currentSection = section;
        currentSectionName = menu.title;

        var $menu = $('<ul class="nav nav-list" />').appendTo($sectionNavigation.empty());
        _.each(menu.topics, function (topicInfo, topicKey) {
            var $link = $('<li><a href="?section='+section+'&topic='+topicKey+'" data-topic="'+topicKey+'">' + topicInfo.title + '</a></li>');
            $menu.append($link);
            if (topicKey) {
                $link.find('a').bind('click', function (evt) {
                    history('pushState', section, topicKey);
                    evt.preventDefault();
                });
            } else {
                $link.addClass('disabled');
            }
        });

        document.title = titlePrefix + currentSectionName;
    }

    function buildSubmenu(section, topic, $rootMenuElement) {
        var topicMenu = sections[section].topics[topic],
            subtopics = topicMenu ? topicMenu.children : [];

        var $subMenu = $('<ul class="nav nav-list" />');
        _.each(subtopics, function (subtopicName, subtopicKey) {
            var $li = $('<li><a data-subtopic="'+subtopicKey+'" href="?section='+section+'&topic='+topic+'&subtopic='+subtopicKey+'">' + subtopicName + '</a></li>');
            $li.find('a').on('click', function (evt) {
                that.switchState(section, topic, subtopicKey);
                evt.preventDefault();
            });
            $subMenu.append($li);
        });
        $rootMenuElement.append($subMenu);
    }

    function runCustomLogic(section, topic) {
        if (section === 'web-usage') {
            $content.find('h1').after('<p>To search, use your browser\'s built-in search, usually Ctrl/Cmd + F</p>');
        }

        var topLinkType = $content.find('h1').length === 0 ? 'h3' : 'h2';
        $content.find(topLinkType).append($('<a href="#" class="top-link">Top</a>').on('click', function (e) { e.preventDefault(); scrollTo(0); }));
    }

    function emptyPage(section, topic) {
        var title = sections[section].topics[topic].title;

        $content.empty().append('<h1>' + title + '</h1><p>Please select a sub-topic on the left for more information!</p>');
    }

    function errorPage(section, topic, subtopic) {
        var title = sections[section].topics[topic].children[subtopic];

        $content.empty().append("<h2>" + title + "</h2><p>It looks like the sub-topic you're looking for does not exist. Please select another topic.</p>");
    }

    function setPage(section, topic, textContent, anchor) {
        var topicName = sections[section].titles[topic];

        $content.empty().append('<div>'+textContent+'</div>');
        loadDemos(section, topic);
        runCustomLogic(section, topic);

        document.title = titlePrefix + currentSectionName + (topicName ? " - " + topicName : '');

        if (anchor && anchor !== '') {
            var $anchor = $('#' + anchor);

            if ($anchor.length) {
                scrollTo($anchor);
            }
        } else {
            scrollTo(0);
        }
    }

    function loadTopic(section, topic, subtopic, anchor) {
        var $sectionRootNavItems = $('#section-navigation-root ul li');
        $sectionRootNavItems.removeClass('active');

        $sectionRootNavItems.removeClass('active').children('ul').remove();
        var $rootMenuElement = $sectionRootNavItems.find('a[data-topic='+topic+']').parent('li').addClass('active');
        buildSubmenu(section, topic, $rootMenuElement);

        if (!subtopic) {
            require(['text!templates/' + section + '/' + topic + '/content.htm'], function (text) {
                setPage(section, topic, text, anchor);
            }, function (err) {
                emptyPage(section, topic);
            });
        } else {
            $sectionRootNavItems.find('a[data-subtopic='+subtopic+']').parent('li').addClass('active');

            require(['text!templates/' + section + '/' + topic + '/' + subtopic + '.htm'], function (text) {
                setPage(section, topic, text, anchor);
            }, function (err) {
                errorPage(section, topic, subtopic);
            });
        }
    }

    function loadHome() {
        require(['text!templates/bluesky.htm'], function (text) {
            $allRootNavItems.removeClass('active');
            $sectionNavigation.empty(); //.removeClass('span2').addClass('span1');
            $content.empty().append($(text));

            document.title = titlePrefix + 'Home';
        });
    }

    function updateQueryString(section, topic, subtopic, anchor) {
        var currentState = History.getState(0);
        if (section !== currentState.data.section || topic !== currentState.data.topic || subtopic !== currentState.data.subtopic || anchor !== currentState.data.anchor) {
            history('replaceState', section, topic, subtopic, anchor);
        }
    }

    that.switchState = function (section, topic, subtopic, anchor) {
        updateQueryString(section, topic, subtopic, anchor);

        if (section === 'home') {
            loadHome();
            return;
        }

        if (section === 'search') {
            currentSection = sections.search;
            currentSectionName = sections.search.title;
            loadTopic('search', 'results', anchor);
            return;
        }

        if (!topic) {
            loadSectionMenuResponse(section, true, sections[section]);
            require(['text!templates/bluesky.htm'], function (text) {
                $content.empty().append($(text));
            });
        } else {
            if (currentSection !== section) {
                loadSectionMenuResponse(section, false, sections[section]);
                loadTopic(section, topic, subtopic, anchor);
            } else {
                loadTopic(section, topic, subtopic, anchor);
            }
        }
    };

    function switchToActiveState () {
        var state = History.getState(),
            urlState = parseURL(state.url);

        if (state.data.section) {
            if (state.data.anchor !== urlState.anchor) {
                state.data.anchor = urlState.anchor;
            }

            that.switchState(state.data.section, state.data.topic, state.data.subtopic, state.data.anchor);
        } else {
            that.switchState(urlState.section || 'home', urlState.topic, urlState.subtopic, urlState.anchor);
        }
    }

    $(function () {
        History.Adapter.bind(window, 'statechange', switchToActiveState);
        switchToActiveState();
    });

    return that;
});
